#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Apr  9 11:58:41 2018

@author: alirezagoudarzi
"""

from __future__ import print_function

# basic torch 
import torch
import torch.nn as nn

import torch.optim as optim
import torch.nn.functional as F
from torch.autograd import Variable

#cudnn for optimized training
import torch.backends.cudnn as cudnn

#torchvision
import torchvision
import torchvision.transforms as transforms

# os utilities
import os
import argparse


# for plotting
import matplotlib
import matplotlib.pyplot as plt
import numpy as np

## webdnn
#from webdnn.frontend.pytorch import PyTorchConverter


# load our models
from model import *

# check for GPU
use_cuda = torch.cuda.is_available()

# set up a transform for pre-processing
transform_train = transforms.Compose([
    #transforms.RandomCrop(32, padding=4),
    #transforms.RandomHorizontalFlip(),
    #transforms.CenterCrop([20,20]),
    #transforms.Scale([14,14]),
    transforms.ToTensor(),
    #transforms.Normalize((0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)),
    
])

transform_test = transforms.Compose([
    transforms.ToTensor(),
    #transforms.Normalize((0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)),
])

trainset = torchvision.datasets.MNIST(
        root='../../../data', train=True,
        download=True, transform=transform_train)
trainloader = torch.utils.data.DataLoader(
        trainset, batch_size=128, shuffle=True, num_workers=2)

testset = torchvision.datasets.MNIST(
        root='../../../data', train=False,
        download=True, transform=transform_test)
testloader = torch.utils.data.DataLoader(
        testset, batch_size=128,
        shuffle=False, num_workers=2)



# instantiate our model 
net = LeNet(BN=False);

# if cuda is available use it
if use_cuda: 
    net.cuda();

# define optimizer for our model
netoptim = optim.Adam(net.parameters(),lr=1e-2)

''' the train function will take care of running a single epoch:
     1) run the model on every batch of data
     2) calculate gradientss
     3) and updating the weights.
     4) calculate train accuracy
'''

# save the loss 
L = [];

def train(): 
    
    # keep track of corrects
    corr = 0;
    # keep track of the number of samples visited
    nums = 0; 
    
    # iterate over batches
    for idx, (input,target) in enumerate(trainloader): 
        
            
        # zero-out the previous gradients (from previous batches)
        netoptim.zero_grad();
        
        # encapsulate input and target data with Variable class. 
        input, target = Variable(input).view(input.size(0),1,28,28), Variable(target)
        
        # if cuda is available use it
        if use_cuda: 
            input, traget = input.cuda(), target.cuda();
        
        # do a forward pass and get the output
        out = net(input);
        # calculate the loss. this is a simple classification so we
        # calculate cross entropy
        loss= F.cross_entropy(out,target);
        
        # run a backward pass (calculates gradients)
        loss.backward();
        L.append(loss.data[0])
        # update the weights
        netoptim.step();
        
        
        # update the number of correct classification
        corr += (out.max(1)[1]==target).float().sum().data[0];
        nums +=  input.size(0); 
        
        # print accuracy every 10 batches
        if idx%10==0: 
            print('acc: %3.2f' % (corr/ nums))
        #if idx>50:
        #    return out, target
    return out,target
        

        

       
for ep in range(1): 
    out,target = train();
    
# bring model back to CPU
net.cpu();
net.training = 0;
# save the parameters to disk
torch.save(net.state_dict(),'NoBN_LeNet.pt')

''' This one is to convert to webdnn
dummy_input = Variable(torch.zeros(1,1,28,28));
graph = PyTorchConverter().convert(net, dummy_input)
exec_info = generate_descriptor("webassembly", graph)  # also "webgpu" "webassembly", "webgl", "fallback" are available.
exec_info.save("./net.webdnn")
'''